# -*- coding: UTF-8 -*-
"""website add pages
"""
import ast
from os.path import isfile, join
from commands import getstatusoutput
from datetime import datetime
from datetime import timedelta
from arv.web.page import Page, usezephir, IPreferences
from arv.db.edge import *
from arv.db.node import *
from arv.lib.factory import get_object_by_id
from arv.lib.util import trace, get_req_archive, gen_archive_name, valid
from arv.lib.logger import logger
from arv.config import nearly_expired_cred, amon_module_list, sphynx_module_list
from zephir.monitor.agents.rvp import parse_ipsec_statusall

# ____________________________________________________________
# save pages root
class SavePage(Page):
    """save pages root page
    """
    save_database = True

# ____________________________________________________________
# add pages
class AddTmplConnect(SavePage):
    def render_page(self, request):
        name = self.get_data_from_request(request=request, arg='name')
        tmpl_node1 = self.get_data_from_request(request=request,
                arg='tmplnode1', object_type='tmpl_node')
        tmpl_node2 = self.get_data_from_request(request=request,
                arg='tmplnode2', object_type='tmpl_node')
        credauth = self.get_data_from_request(request=request,
                arg='credauth', object_type='credential')
        leftsendcert = self.get_data_from_request(request=request,
                arg='leftsendcert')
        if not credauth.ca:
            raise Exception('Credential need to be authority')
        add_tmpl_connect(name, tmpl_node1, tmpl_node2, credauth, leftsendcert)
        return "TmplConnect added"

class AddTmplNode(SavePage):
    def render_page(self, request):
        name = request.args['name'][0]
        add_tmpl_node(name)
        return 'TmplNode added'

class AddTmplEdge(SavePage):
    def render_page(self, request):
        name = self.get_data_from_request(request=request, arg='name')
        tmpl_vertex1 = self.get_data_from_request(request=request,
                arg='tmplvertex1', object_type='tmpl_vertex')
        tmpl_vertex2 = self.get_data_from_request(request=request,
                arg='tmplvertex2', object_type='tmpl_vertex')
        tmpl_connect = self.get_data_from_request(request=request,
                arg='id', object_type='tmpl_connect')

        tmpl_connect.add_tmpl_edge(name, tmpl_vertex1, tmpl_vertex2)
        return 'TmplEdge added'

class AddTmplVertex(SavePage):
    def render_page(self, request):
        name = self.get_data_from_request(request=request, arg='name')
        tmpl_node = self.get_data_from_request(request=request,
                arg='id', object_type='tmpl_node')
        mimetype = self.get_data_from_request(request=request,
                arg='mimetype')
        zephir_module = self.get_data_from_request(request=request,
                arg='moduleid')
        zephir_var_ip1 = self.get_data_from_request(request=request, arg='ip1')
        zephir_var_ip2 = self.get_data_from_request(request=request,
                arg='ip2', required = False)

        tmpl_vertex1 = tmpl_node.add_tmpl_vertex(name=name,
                mimetype=mimetype,
                zephir_module=zephir_module,
                zephir_var_ip1=zephir_var_ip1,
                zephir_var_ip2=zephir_var_ip2)
        return 'TmplVertex added'


class AddNode(SavePage):
    @trace()
    def render_page(self, request):
        name =  self.get_data_from_request(request=request, arg='name')
        uai =  self.get_data_from_request(request=request, arg='uai')
        try:
            id_zephir =  int(self.get_data_from_request(request=request, arg='id_zephir'))
        except:
            # In manual mode --> no id_zephir
            id_zephir = None
        try:
            eole_version =  self.get_data_from_request(request=request, arg='eole_version')
        except:
            # In manual mode --> no eole_version
            eole_version = None
        tmpl_node = self.get_data_from_request(request=request, arg='id', object_type='tmpl_node', required=False)
        mimetype  = self.get_data_from_request(request=request, arg='mimetype', required=False)
        logger.debug('#-> add_node param :name: ' + name)
        if tmpl_node == None:
            if not mimetype:
                mimetype = u'etablissement'
            tmpl_node = get_tmpl_nodes(mimetype=mimetype)[0]
        vertices = self.get_data_from_request(request=request, arg='vertices', load_json=True)

        node = tmpl_node.add_node(name=name, uai=uai, id_zephir=id_zephir, eole_version=eole_version)
        for tvid, ip in vertices.items():
            logger.debug('#ip : %s'%ip)
            ip1 = ip[0]
            ip2 = ip[1]
            if not ip1 in ["", None] and not ip2 in ["", None]:
                logger.debug('add this ip')
                tmpl_vertex = get_object_by_id(int(tvid), 'tmpl_vertex')
                if ip2 == "":
                    ip2 = None
                tmpl_vertex.add_vertex(ip1=ip1, ip2=ip2, node=node)
        return 'Node added'

class AddConnect(SavePage):
    def render_page(self, request):
        tail_node = self.get_data_from_request(request=request, arg='node1', object_type='node')
        head_node = self.get_data_from_request(request=request, arg='id', object_type='node')

        tail_extr = self.get_data_from_request(request=request, arg='extr1', object_type='extremity')
        head_extr = self.get_data_from_request(request=request, arg='extr2', object_type='extremity')

        tail_cred = self.get_data_from_request(request=request, arg='cred1', object_type='credential')
        head_cred = self.get_data_from_request(request=request, arg='cred2', object_type='credential')

        leftsendcert = self.get_data_from_request(request=request, arg='leftsendcert')

        tmpl_connect = self.get_data_from_request(request=request, arg='tmpl_connect', object_type='tmpl_connect')
        edges = self.get_data_from_request(request=request, arg='edges', load_json=True)

        connect = tmpl_connect.add_connect(tail_node, head_node,
                tail_extr, head_extr, tail_cred, head_cred, leftsendcert)
        for tmpledgeid, activated in edges.items():
            if activated:
                tmpl_edge = get_object_by_id(tmpledgeid, 'tmpl_edge')
                tmpl_edge.add_edge(connect)
        return 'Connect added'

class AddImportedCred(SavePage):
    def render_page(self, request):
        private_key = self.get_data_from_request(request=request, arg='private_key')
        public_key = self.get_data_from_request(request=request, arg='public_key')
        password = self.get_data_from_request(request=request, arg='password', required=True)
        node = self.get_data_from_request(request=request, arg='node', object_type='node')
        node.import_credential(private_key=private_key, credential=public_key, passwd=password)
        return 'Credential imported'

class AddGeneratedCred(SavePage):
    def render_page(self, request):
        name = self.get_data_from_request(request=request, arg='name')
        passwd = self.get_data_from_request(request=request, arg='passwd')
        node = self.get_data_from_request(request=request, arg='node', object_type='node')
        #cred_priv_type = self.get_data_from_request(request=request, arg='cred_priv_type')
        cred_priv_type = request.args['cred_priv_type'][0]
        logger.debug("#-> cred_priv_type" + cred_priv_type)
        credential = add_credential(name, passwd, node, cred_priv_type)
        if credential != True:
            return ("download_name", 'arv/dl_credential?name=%s'%name, 'Credential added')
        return 'Credential added'

class DlCredential(SavePage):
    def render_page(self, request):
        #return ("download_file", 'toto.cert', "ca c'est du gros fichier.")
        name = self.get_data_from_request(request=request, arg='name')
        #logger.debug('unsigned_credential in DlCredential: %s'%str(credential.unsigned_credential))
        reqfile_content = get_req_archive(name)
        return ("download_file", name+".tgz", reqfile_content) #credential.unsigned_credential)

class AddCredAuth(SavePage):
    def render_page(self, request):
        credentialauth = self.get_data_from_request(request=request, arg='credentialauth')
        add_credential_auth(credential=credentialauth)
        return 'Credential auth added'

class AddExtremity(SavePage):
    def render_page(self, request):
        pub_ip = request.args['pub_ip'][0]
        priv_ip = request.args['priv_ip'][0]
        if priv_ip == "":
            priv_ip = None
        node = self.get_data_from_request(request=request, arg='id', object_type='node')
        node.add_extremity(pub_ip=pub_ip, priv_ip=priv_ip)
        return 'Extremity added'

# ____________________________________________________________
# mod pages

class ModNode(SavePage):
    @trace()
    def render_page(self, request):
        node = self.get_data_from_request(request=request, arg='node',  object_type='node')
        vertices = self.get_data_from_request(request=request, arg='vertices', load_json=True)
        node_vertices = {}
        for vertex in node.vertices:
            if vertex.tmpl_vertex:
                node_vertices[vertex.tmpl_vertex.id] = vertex
        mod_node = []
        for tvid, ip in vertices.items():
            ip1 = ip[0]
            ip2 = ip[1]
            if ip2 == "":
                ip2 = None
            if not ip1 in ["", None]:
                tvid = int(tvid)
                if not tvid in node_vertices:
                    tmpl_vertex = get_object_by_id(tvid, 'tmpl_vertex')
                    tmpl_vertex.add_vertex(ip1=ip1, ip2=ip2, node=node)
                else:
                    node_vertices[tvid].set_ips(ip1, ip2)
                    mod_node.append(tvid)
            #if ip1 is empty, do nothing
        for tvid in list(set(node_vertices.keys()) - set(mod_node)):
            logger.debug('#suppression de tvid: %s' % tvid)
            del_vertex(node_vertices[tvid])
        return 'Node modified'

class ModTmplEdge(SavePage):
    def render_page(self, request):
        tmpl_edge = self.get_data_from_request(request=request, arg='id', object_type='tmpl_edge')
        name = self.get_data_from_request(request=request, arg='name')
        tmpl_vertex1 = self.get_data_from_request(request=request, arg='tmplvertex1', object_type='tmpl_vertex')
        tmpl_vertex2 = self.get_data_from_request(request=request, arg='tmplvertex2', object_type='tmpl_vertex')

        tmpl_edge.mod_name(name)
        tmpl_edge.mod_tmpl_vertex1(tmpl_vertex1)
        tmpl_edge.mod_tmpl_vertex2(tmpl_vertex2)
        return 'TmplEdge modified'

class ModTmplVertex(SavePage):
    def render_page(self, request):
        tmpl_vertex = self.get_data_from_request(request=request, arg='tmpl_vertex_id', object_type='tmpl_vertex')
        name = self.get_data_from_request(request=request, arg='name')
        tmpl_node = self.get_data_from_request(request=request, arg='id', object_type='tmpl_node')
        mimetype = self.get_data_from_request(request=request, arg='mimetype')
        zephir_module = self.get_data_from_request(request=request, arg='moduleid')
        zephir_var_ip1 = self.get_data_from_request(request=request, arg='ip1')
        zephir_var_ip2 = self.get_data_from_request(request=request, arg='ip2', required = False)

        tmpl_vertex.mod_name(name)
        tmpl_vertex.mod_mimetype(mimetype)
        tmpl_vertex.mod_zephir(zephir_module, zephir_var_ip1, zephir_var_ip2)

        return 'TmplVertex modified'

class ModConnect(SavePage):
    @trace()
    def render_page(self, request):
        connect = self.get_data_from_request(request=request, arg='connect_id', object_type='connect')
        tail_extr = self.get_data_from_request(request=request, arg='extr1', object_type='extremity')
        head_extr = self.get_data_from_request(request=request, arg='extr2', object_type='extremity')
        tail_cred = self.get_data_from_request(request=request, arg='cred1', object_type='credential')
        head_cred = self.get_data_from_request(request=request, arg='cred2', object_type='credential')
        leftsendcert = self.get_data_from_request(request=request, arg='leftsendcert')
        edges = self.get_data_from_request(request=request, arg='edges', load_json=True)

        if connect.tail_node == tail_extr.node:
            connect.mod_tail_extr(tail_extr)
            connect.mod_head_extr(head_extr)
            connect.mod_tail_cred(tail_cred)
            connect.mod_head_cred(head_cred)
        else:
            connect.mod_tail_extr(head_extr)
            connect.mod_head_extr(tail_extr)
            connect.mod_tail_cred(head_cred)
            connect.mod_head_cred(tail_cred)
        connect.leftsendcert = leftsendcert

        current_edges = {}
        for edge in connect.edges:
            current_edges[edge.tmpl_edge.id] = edge
        current_tmpledges_id = current_edges.keys()
        for tmpledgeid, activated in edges.items():
            if activated:
                #if activated and not in current connect, add it
                if tmpledgeid not in current_tmpledges_id:
                    tmpl_edge = get_object_by_id(tmpledgeid, 'tmpl_edge')
                    tmpl_edge.add_edge(connect)
            else:
                #if not activated and in current connect, remove it
                if tmpledgeid in current_tmpledges_id:
                    connect.edges.pop(current_edges[tmpledgeid])
        #remove edges in current connect not in new connect
        for extra_id in list(set(current_tmpledges_id)-set(edges.keys())):
            connect.edges.remove(current_edges[extra_id])
            current_edges[extra_id].delete()

        return 'Connect modified'

class ModKeys(SavePage):
    @trace()
    def render_page(self, request):
        cred = self.get_data_from_request(request=request, arg='cred_id',
                object_type='credential')
        private_key = self.get_data_from_request(request=request,
                arg='private_key', required=False)
        public_key = self.get_data_from_request(request=request,
                arg='public_key')
        password = self.get_data_from_request(request=request,
                arg='password')
        cred.mod_keys(public_key, password, private_key)

class ModCA(SavePage):
    @trace()
    def render_page(self, request):
        cred = self.get_data_from_request(request=request, arg='cred_id',
                object_type='credential')
        credential = self.get_data_from_request(request=request,
                arg='credential')
        cred.mod_ca(credential)
# ____________________________________________________________
# del pages

class DelTmplConnect(SavePage):
    def render_page(self, request):
        tmpl_connect = self.get_data_from_request(request=request,
                arg='deleteid', object_type='tmpl_connect')
        del_tmpl_connect(tmpl_connect)

        return ''

class DelTmplNode(SavePage):
    def render_page(self, request):
        tmpl_node = self.get_data_from_request(request=request, arg='deleteid',
                object_type='tmpl_node')
        del_tmpl_node(tmpl_node)

        return ''

class DelTmplEdge(SavePage):
    def render_page(self, request):
        tmpl_edge = self.get_data_from_request(request=request, arg='deleteid',
                object_type='tmpl_edge')
        del_tmpl_edge(tmpl_edge)

        return ''

class DelTmplVertex(SavePage):
    def render_page(self, request):
        tmpl_vertex = self.get_data_from_request(request=request, arg='deleteid', object_type='tmpl_vertex')
        del_tmpl_vertex(tmpl_vertex)

        return ''

class DelNode(SavePage):
    def render_page(self, request):
        node = self.get_data_from_request(request=request, arg='deleteid', object_type='node')
        del_node(node)
        return 'Node deleted'

class DelEdge(SavePage):
    def render_page(self, request):
        obj = self.get_data_from_request(request=request, arg='deleteid', object_type='edge')
        connect = obj.connect
        del_conn = False
        if connect.edges == [obj]:
            del_conn = True
        del_edge(obj)
        if del_conn:
            del_connect(connect)
            return 'Edge and connect deleted'
        return 'Edge deleted'

class DelConnect(SavePage):
    def render_page(self, request):
        connect = self.get_data_from_request(request=request, arg='deleteid', object_type='connect')
        for edge in connect.edges:
            del_edge(edge)
        del_connect(connect)
        return 'Connect deleted'

class DelCred(SavePage):
    def render_page(self, request):
        obj = self.get_data_from_request(request=request, arg='deleteid', object_type='credential')
        del_cred(obj)
        return 'Credential deleted'

class DelExtrIP(SavePage):
    def render_page(self, request):
        obj = self.get_data_from_request(request=request, arg='deleteid', object_type='extremity')
        del_extremity(obj)
        return 'Extremity deleted'

class ModExtrIP(SavePage):
    def render_page(self, request):
        extremity = self.get_data_from_request(request=request, arg='id', object_type='extremity')
        pub_ip = request.args['pub_ip'][0]
        priv_ip = request.args['priv_ip'][0]
        if priv_ip == "":
            priv_ip = None
        extremity.pub_ip = pub_ip
        extremity.priv_ip = priv_ip
        return 'Extremity modified'

# ____________________________________________________________
# get pages
class GetNodes(Page):
    def render_page(self, request):
        ret = []
        connected = self.get_data_from_request(request=request, arg='connected', required=False)
        if connected == 'true':
            nodea = self.get_data_from_request(request=request, arg='id', object_type='node')
            nodes = get_connected_nodes(nodea)
        else:
            nodea = self.get_data_from_request(request=request, arg='id', object_type='node', required=False)
            nodes = get_nodes(nodea)
        for node in nodes:
            ret.append({'uai': node.uai, 'node': node.name, 'id': node.id})
        return ret

class GetNodesState(Page):
    def render_page(self, request):
        ret = []
        nodes = get_nodes()
        if isfile('/tmp/tunnels_status.txt'):
            try:
                tunnels_status = open('/tmp/tunnels_status.txt', "r")
                tunnels = tunnels_status.readline()
                tunnels = ast.literal_eval(str(tunnels))
                total = int(tunnels_status.readline())
                up = int(tunnels_status.readline())
                down = int(tunnels_status.readline())
            except:
                tunnels = []
                total = 0
                up = 0
                down = 0
        else:
            tunnels = []
            total = 0
            up = 0
            down = 0

        tmpl_nodes = get_tmpl_nodes(u'sphynx')
        current_node = tmpl_nodes[0].nodes[0]
        for node in nodes:
            if node == current_node:
                state = ""
                ret.append({
                    'node': node.name,
                    'id': node.id,
                    'id_zephir': node.id_zephir,
                    'eole_version':node.eole_version,
                    'state': state,
                    'uai': node.uai
                })
                continue
            state = 'Pb de tunnels ou non configuré'
            for tunnel in tunnels:
                if tunnel['child_name'] == '':
                    bad_child = 0
                    ip_dst = tunnel['dst'].split('[')[0]
                    ip_src = tunnel['src'].split('[')[0]
                    ip_in_extrimities = False
                    for extremity in node.extremities:
                        if valid(ip_dst, 'ip') == extremity.pub_ip or valid(ip_dst, 'ip') == extremity.priv_ip:
                            ip_in_extrimities = True
                        elif valid(ip_src, 'ip') == extremity.pub_ip or valid(ip_dst, 'ip') == extremity.priv_ip:
                            ip_in_extrimities = True
                        if node.name in unicode(tunnel['peer_name'], 'utf-8') and ip_in_extrimities:
                            if tunnel['status'] == 'On':
                                state = 'Connexion OK'
                            else:
                                state = 'Problème de connexion'
                if node.name in unicode(tunnel['peer_name'], 'utf-8') and tunnel['child_name'] != '':
                    if tunnel['status'] == 'Off' and bad_child == 0:
                        state += ' - Problème de tunnel(s)'
                        bad_child +=1

            ret.append({
                'node': node.name,
                'id': node.id,
                'id_zephir': node.id_zephir,
                'eole_version':node.eole_version,
                'state': state,
                'uai': node.uai
            })
        return ret

class GetEdges(Page):
    def render_page(self, request):
        ret = []
        nodea = self.get_data_from_request(request=request, arg='nodea', object_type='node')
        nodeb = self.get_data_from_request(request=request, arg='nodeb', object_type='node')
        for connect in get_connects(nodea, nodeb):
            if connect.edges == []:
                ret.append({'connect': connect.tmpl_connect.name})
            else:
                connect_infos = connect.tmpl_connect.name
                if connect.head_extr.node == nodea:
                    head_extr = connect.head_extr
                elif connect.head_extr.node == nodeb:
                    tail_extr = connect.head_extr
                if connect.tail_extr.node == nodea:
                    head_extr = connect.tail_extr
                elif connect.tail_extr.node == nodeb:
                    tail_extr = connect.tail_extr
                if head_extr.pub_ip == head_extr.priv_ip:
                    connect_infos += " - {0}".format(str(head_extr.pub_ip))
                else:
                    connect_infos += " - {0} --> {1}".format(str(head_extr.pub_ip), str(head_extr.priv_ip))
                if tail_extr.pub_ip == tail_extr.priv_ip:
                    connect_infos += " ... {0}".format(str(tail_extr.pub_ip))
                else:
                    connect_infos += " ... {0} <-- {1}".format(str(tail_extr.priv_ip), str(tail_extr.pub_ip))
                connect_infos += " / " + " certificate send : " + connect.leftsendcert.upper() + " via ipsec protocol"
                for edge in connect.edges:
                    if edge.tmpl_edge.tmpl_vertex_a.tmpl_node == nodea.tmpl_node:
                        vertexa = edge.tmpl_edge.tmpl_vertex_a.name
                        vertexb = edge.tmpl_edge.tmpl_vertex_b.name
                    else:
                        vertexa = edge.tmpl_edge.tmpl_vertex_b.name
                        vertexb = edge.tmpl_edge.tmpl_vertex_a.name
                    for vertex in edge.tmpl_edge.tmpl_vertex_a.vertices:
                        if vertex.node == nodea:
                            if vertex.tmpl_vertex.mimetype == u'network':
                                sep = " / "
                            elif vertex.tmpl_vertex.mimetype == u'ip':
                                sep = ""
                            elif vertex.tmpl_vertex.mimetype == u'range':
                                sep = " ... "
                            vertexa += " : {0}{1}{2}".format(vertex.ip1, sep, '' if vertex.ip2 is None else vertex.ip2)
                        elif vertex.node == nodeb:
                            if vertex.tmpl_vertex.mimetype == u'network':
                                sep = " / "
                            elif vertex.tmpl_vertex.mimetype == u'ip':
                                sep = ""
                            elif vertex.tmpl_vertex.mimetype == u'range':
                                sep = " ... "
                            vertexb += " : {0}{1}{2}".format(vertex.ip1, sep, '' if vertex.ip2 is None else vertex.ip2)
                    for vertex in edge.tmpl_edge.tmpl_vertex_b.vertices:
                        if vertex.node == nodea:
                            if vertex.tmpl_vertex.mimetype == u'network':
                                sep = " / "
                            elif vertex.tmpl_vertex.mimetype == u'ip':
                                sep = ""
                            elif vertex.tmpl_vertex.mimetype == u'range':
                                sep = " ... "
                            vertexa += " : {0}{1}{2}".format(vertex.ip1, sep, '' if vertex.ip2 is None else vertex.ip2)
                        elif vertex.node == nodeb:
                            if vertex.tmpl_vertex.mimetype == u'network':
                                sep = " / "
                            elif vertex.tmpl_vertex.mimetype == u'ip':
                                sep = ""
                            elif vertex.tmpl_vertex.mimetype == u'range':
                                sep = " ... "
                            vertexb += " : {0}{1}{2}".format(vertex.ip1, sep, '' if vertex.ip2 is None else vertex.ip2)
                    ret.append({'edge': edge.tmpl_edge.name, 'id': edge.id,
                            'connect': connect_infos,
                            'vertexa': vertexa , 'vertexb': vertexb,
                            'activate': False})
        return ret

class GetConnects(Page):
    def render_page(self, request):
        ret = []
        nodea = self.get_data_from_request(request=request, arg='nodea', object_type='node')
        nodeb = self.get_data_from_request(request=request, arg='nodeb', object_type='node')
        for connect in get_connects(nodea, nodeb):
            ret.append({'id': connect.id, 'connect': connect.tmpl_connect.name, 'leftsendcert': connect.leftsendcert})
        return ret

class GetTmplNodes(Page):
    def render_page(self, request):
        ret = []
        exclude = self.get_data_from_request(request=request, arg='exclude', object_type='tmpl_node', required=False)
        mimetype = self.get_data_from_request(request=request, arg='mimetype', required=False)
        for tmplnode in get_tmpl_nodes(mimetype=mimetype):
            if tmplnode != exclude:
                ret.append({'tmplnode': tmplnode.name, 'id': tmplnode.id, 'mimetype': tmplnode.mimetype})
        return ret

class GetTmplConnects(Page):
    def render_page(self, request):
        ret = []
        node_a = self.get_data_from_request(request=request, arg='node_a_id', object_type='node', required=False)
        if node_a != None:
            node_b = self.get_data_from_request(request=request, arg='node_b_id', object_type='node')
            tmpl_connects = []
            for connect in get_connects(node_a, node_b):
                tmpl_connects.append(connect.tmpl_connect)
            tmpl_node_a = node_a.tmpl_node
            tmpl_node_b = node_b.tmpl_node
        else:
            tmpl_node_a= None
            tmpl_node_b = None
            tmpl_connects = []
        for tmpl_connect in get_tmpl_connects(tmpl_node_a, tmpl_node_b):
            if tmpl_connect not in tmpl_connects:
                ret.append({'tmplconnect': tmpl_connect.name,
                        'id': tmpl_connect.id,
                        'tmplnodea': tmpl_connect.tmpl_node_a.name,
                        'tmplnodeaid': tmpl_connect.tmpl_node_a.id,
                        'tmplnodeb': tmpl_connect.tmpl_node_b.name,
                        'tmplnodebid': tmpl_connect.tmpl_node_b.id,
                        'leftsendcert': tmpl_connect.leftsendcert})
        return ret

class GetTmplEdges(Page):
    def render_page(self, request):
        ret = []
        tmpl_connect = self.get_data_from_request(request=request, arg='id', object_type='tmpl_connect', required=False)
        connect = self.get_data_from_request(request=request, arg='connect_id', object_type='connect', required=False)
        tmpl_edges = []
        if connect != None:
            for edge in connect.edges:
               tmpl_edges.append(edge.tmpl_edge)
            if tmpl_connect == None:
                tmpl_connect = connect.tmpl_connect
        for tmpledge in get_tmpl_edges(tmpl_connect):
            rettedge = {'id': tmpledge.id,
                'tmplvertexname': tmpledge.name,
                'tmplvertex1': tmpledge.tmpl_vertex_a.name,
                'tmplvertex1id': tmpledge.tmpl_vertex_a.id,
                'tmplvertex2': tmpledge.tmpl_vertex_b.name,
                'tmplvertex2id': tmpledge.tmpl_vertex_b.id,
            }
            if tmpledge in tmpl_edges:
                rettedge['activate'] = True
            ret.append(rettedge)
        return ret

class GetTmplVertices(Page):
    @trace()
    def render_page(self, request):
        ret = []
        tmpl_node = self.get_data_from_request(request=request, arg='tmplnodeid', object_type='tmpl_node', required=False)
        node = self.get_data_from_request(request=request, arg='nodeid', object_type='node', required=False)
        uai = self.get_data_from_request(request=request, arg='uai', required=False)
        name = self.get_data_from_request(request=request, arg='name', required=False)
        force_zephir = self.get_data_from_request(request=request, arg='force_zephir', required=False)
        modtmplvertex = self.get_data_from_request(request=request, arg='modtmplvertex', required=False)
        #if None not in [tmpl_node, node] or tmpl_node == node:
        #    raise Exception('Must specified tmpl_node or node')
        if node != None:
            tmpl_node = node.tmpl_node
            tmpl_vertex = {}
            for vertex in node.vertices:
                tmpl_vertex[vertex.tmpl_vertex] = (vertex.ip1, vertex.ip2)
        if tmpl_node != None:
            for tmplvertex in get_tmpl_vertices(tmpl_node):
                ip1 = ''
                ip2 = ''
                module = ''
                if node != None and force_zephir != 'true':
                    # Charge les valeurs des vertex d'un node
                    ip1, ip2 = tmpl_vertex.get(tmplvertex, ('', ''))
                else:
                    if uai != None:
                        # Recharge les infos Zéphir et tmplvertex
                        prefs = request.getSession(IPreferences)
                        if prefs.zephir != None:
                            # Si connecté avec user Zéphir
                            if tmplvertex.zephir_module:
                                # Recharge les valeur depuis Zéphir
                                ip1 = prefs.zephir.get_var(
                                        uai=uai,
                                        name=name,
                                        module=tmplvertex.zephir_module,
                                        var=tmplvertex.zephir_var_ip1
                                )
                                if tmplvertex.mimetype != u'ip':
                                    ip2 = prefs.zephir.get_var(
                                            uai=uai,
                                            name=name,
                                            module=tmplvertex.zephir_module,
                                            var=tmplvertex.zephir_var_ip2)
                            else:
                                # Recharge les valeur depuis tmplvertex
                                ip1 = tmplvertex.zephir_var_ip1
                                ip2 = tmplvertex.zephir_var_ip2
                        else:
                            # Si connecté avec user system
                            if not tmplvertex.zephir_module:
                                ip1 = tmplvertex.zephir_var_ip1
                                ip2 = tmplvertex.zephir_var_ip2
                    else:
                        #if search tmplVertex attribute
                        module = tmplvertex.zephir_module
                        if modtmplvertex == 'true' or not module:
                            # Si modif tmplvertex
                            # ou si ajout manuel serveur
                            ip1 = tmplvertex.zephir_var_ip1
                            ip2 = tmplvertex.zephir_var_ip2

                    if (ip1, ip2) in [('', ''), (None, None)] and \
                            force_zephir == 'true':
                        #if force_zephir is set and no value from Zéphir
                        ip1, ip2 = tmpl_vertex.get(tmplvertex, ('', ''))

                ret.append({'tmplvertex': tmplvertex.name,
                    'id': tmplvertex.id,
                    'mimetype': tmplvertex.mimetype,
                    'module': module,
                    'ip1': ip1,
                    'ip2': ip2
                })
        else:
            for tmplvertex in get_tmpl_vertices():
                ret.append({'tmplvertexname': tmplvertex.name,
                    'id': tmplvertex.id,
                    'tmplvertexmimetype': tmplvertex.mimetype,
                    'tmplnode': tmplvertex.tmpl_node.name,
                    'tmplnodeid': tmplvertex.tmpl_node.id,
                    'tmplvertexmodule': tmplvertex.zephir_module,
                    'tmplvertexip1': tmplvertex.zephir_var_ip1,
                    'tmplvertexip2': tmplvertex.zephir_var_ip2,
                })

        return ret

class GetCred(Page):
    def render_page(self, request):
        ret = []
        node = self.get_data_from_request(request=request, arg='node', object_type='node')
        connect = self.get_data_from_request(request=request, arg='connect_id', object_type='connect', required=False)
        tmpl_connect = self.get_data_from_request(request=request, arg='tmplconnect_id', object_type='tmpl_connect', required=False)
        if None not in [connect, tmpl_connect]:
            raise Exception('Must specified tmpl_connect or connect')
        connect_cred = None
        cred_auth = None
        if connect != None:
            if node == connect.tail_node:
                connect_cred = connect.tail_cred.id
            elif node == connect.head_node:
                connect_cred = connect.head_cred.id
            else:
                raise Exception('Node is not in connect')
            cred_auth = connect.tmpl_connect.cred_auth
        if tmpl_connect != None:
            cred_auth = tmpl_connect.cred_auth

        for cred in node.get_credentials(cred_auth):
            retcred = {'id':cred.id, 'cred':cred.name}
            if retcred['id'] == connect_cred:
                retcred['selected'] = True
            ret.append(retcred)
        return ret

class GetCredAuth(Page):
    def render_page(self, request):
        ret = []
        for credauth in get_credentials_auth():
            ret.append({'id':credauth.id, 'credauth':credauth.name})
        return ret

class GetAllCred(Page):
    def render_page(self, request):
        def sort_cred(x, y):
            return cmp(x['delta'], y['delta'])
        ret = []
        for cred in get_all_credentials():
            today = datetime.now()
            expiration_date = datetime.strptime(cred.expiration_date, '%Y/%m/%d')
            delta = expiration_date - today
            delta = delta.days
            if nearly_expired_cred > delta:
                is_nearly_expired = True
            else:
                is_nearly_expired = False
            if 0 > delta:
                is_expired = True
            else:
                is_expired = False
            related_node_name = ""
            related_node = cred.node
            if related_node:
                uai = related_node.uai if related_node.uai is not None else ""
                name = related_node.name if related_node.name is not None else ""
                related_node_name = uai + " - " + name
            elif not cred.ca:
                related_node_name = "PAS DE SERVEUR RVP ASSOCIÉ !!!"
            ret.append({'id':cred.id, 'name': cred.name,
                'ca': cred.ca, 'expiration_date': cred.expiration_date,
                'delta': delta, 'is_expired': is_expired,
                'is_nearly_expired': is_nearly_expired,
                'node': related_node_name})
        ret.sort(sort_cred)
        return ret

class GetExtr(Page):
    def render_page(self, request):
        ret = []
        node = self.get_data_from_request(request=request, arg='node',
                object_type='node', required=False)
        connect = self.get_data_from_request(request=request, arg='connect_id',
                object_type='connect', required=False)
        extrip = self.get_data_from_request(request=request, arg='extrip',
                object_type="extremity", required=False)
        new = self.get_data_from_request(request=request, arg='new',
                required=False)
        if not None in [node, extrip] or node == extrip == None:
            raise Exception('il faut obligatoirement un node ou une extremity mais jamais les deux')
        if extrip:
            return {'id': extrip.id, 'pub_ip': extrip.pub_ip, 'priv_ip': extrip.priv_ip}
        else:
            if new == 'true':
                ret = {'pub_ip': '', 'priv_ip': ''}
                if usezephir == True:
                    prefs = request.getSession(IPreferences)
                    if prefs.zephir != None:
                        try:
                            haute_dispo = prefs.zephir.get_var(
                                    uai=node.uai,
                                    name=node.name,
                                    var='activer_haute_dispo')
                            if haute_dispo == "maitre":
                                vips_name = prefs.zephir.get_var(
                                    uai=node.uai,
                                    name=node.name,
                                    var="vip_resource_name").split('| ')
                                vips_adresseip = prefs.zephir.get_var(
                                    uai=node.uai,
                                    name=node.name,
                                    var="vip_resource_adresseip").split('| ')
                                try:
                                    index = vips_name.index('VIP_externe')
                                    ip = vips_adresseip[index]
                                except:
                                    logger.debug("VIP_externe resource name does not exist")
                                    raise Exception("VIP_externe resource name does not exist")
                            elif haute_dispo == "non":
                                logger.debug("Pas de haute dispo")
                                ip = prefs.zephir.get_var(
                                    uai=node.uai,
                                    name=node.name,
                                    var='adresse_ip_eth0')
                            else:
                                logger.debug("Pas de haute dispo")
                                ip = prefs.zephir.get_var(
                                    uai=node.uai,
                                    name=node.name,
                                    var='adresse_ip_eth0')
                        except:
                            ip = "1.1.1.1"
                            logger.debug("No value found from Zéphir, returns 1.1.1.1")
                        iptype = IP(ip).iptype()
                        if iptype == 'PRIVATE':
                            ret['priv_ip'] = ip
                        elif iptype == 'PUBLIC':
                            ret['pub_ip'] = ip
            else:
                connect_extr = None
                if connect != None:
                    if node == connect.tail_node:
                        connect_extr = connect.tail_extr.id
                    elif node == connect.head_node:
                        connect_extr = connect.head_extr.id
                    else:
                        raise Exception('Node is not in connect')
                for extr in node.get_extremities():
                    retextr = {'id': extr.id, 'extr': extr.pub_ip}
                    if connect_extr != None:
                        if retextr['id'] == connect_extr:
                            retextr['selected'] = True
                    ret.append(retextr)
            return ret

class GetZephirEtabs(Page):
    @trace()
    def render_page(self, request):
        ret = []
        if usezephir == True:
            prefs = request.getSession(IPreferences)
            if prefs.zephir != None:
                aetab = prefs.zephir.get_amons()
                aetab.extend(prefs.zephir.get_sphynxs())
                logger.debug("#GetZephirEtabs: {0}".format(aetab))
                zetab = [(libelle, uai) for libelle, uai, id_zephir, eole_version, mimetype in aetab]
                letab = [(libelle, uai) for libelle, uai, id_zephir, eole_version, mimetyoe in get_etabs()]
                zid_zephir = [(id_zephir) for libelle, uai, id_zephir, eole_version, mimetype in aetab]
                lid_zephir = [(id_zephir) for libelle, uai, id_zephir, eole_version, mimetyoe in get_etabs()]
                keyword = ('libelle', 'uai', 'id_zephir', 'eole_version', 'mimetype')
                #return only zetab not in letab and id_zephir not already extsts
                for etab in [(libelle, uai, id_zephir, eole_version, mimetype) for libelle, uai, id_zephir, eole_version, mimetype in aetab if (libelle, uai) in list(set(zetab) - set(letab)) and id_zephir in list(set(zid_zephir) - set(lid_zephir))]:
                    ret.append(dict(zip(keyword, etab)))
        return ret

class GetZephirModules(Page):
    def render_page(self, request):
        ret = []
        if usezephir == True:
            prefs = request.getSession(IPreferences)
            if prefs.zephir != None:
                 ret = prefs.zephir.get_modules()
        #Insert empty field
        ret.insert(0, {'name': None, 'id': None})
        return ret

class GetZephirVariables(Page):
    def render_page(self, request):
        module = self.get_data_from_request(request=request, arg='module')
        if module == '':
            return []
        ret = []
        if usezephir == True:
            prefs = request.getSession(IPreferences)
            if prefs.zephir != None:
                ret = prefs.zephir.get_variables(module)
        return ret

class SynchroARVZephir(SavePage):
    def render_page(self, request):
        ret = []
        if usezephir == True:
            node = self.get_data_from_request(request=request, arg='node',
                    object_type='node')
            prefs = request.getSession(IPreferences)
            if prefs.zephir != None:
                uai = node.uai
                name = node.name
                id_zephir = node.id_zephir
                server = prefs.zephir.get_etab_server(uai, name, id_zephir)
                logger.debug(str(server))
                if server:
                    if id_zephir is not None:
                        logger.debug("Update uai and name from Zéphir")
                        uai = server['rne']
                        libelle = server['libelle']
                        if type(uai) == unicode:
                            node.uai = str(unicode.encode(uai, "utf-8")).decode('utf-8')
                        else:
                            node.uai = uai
                        if type(libelle) == unicode:
                            node.name = str(unicode.encode(libelle, 'utf-8')).decode('utf-8')
                        else:
                            node.name = libelle
                    else:
                        logger.debug("Fill id_zephir and eole_version")
                        node.id_zephir = int(server['id'])
                        module_eole_version = prefs.zephir.dict_eole_version(amon_module_list + sphynx_module_list)
                        node.eole_version = module_eole_version[server['module_actuel']]
                else:
                    logger.debug("Node does not exist in Zéphir")
        else:
            logger.debug("No Zéphir connection")
        return "Node modified"

class SendDb(Page):
    def render_page(self, request):
        node = self.get_data_from_request(request=request, arg='node',
                object_type='node')
        if usezephir == True:
            prefs = request.getSession(IPreferences)
            if prefs.zephir != None:
                uai = node.uai
                name = node.name
                id_zephir = node.id_zephir
                amonpath, archivename = gen_archive_name(uai, name)
                archive = join(amonpath, archivename)
                prefs.zephir.send_db(archive, uai, name, id_zephir)
                return "Resended"

        return Exception('Not loggin to Zephir')

